# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import MessageV2
from driverng.splitted import Date, Splitted, String


class QCMLResult(Splitted):
    SEPARATOR = "|"
    FIELDS = (
        String("instrument_id"),
        String("lot"),
        Date("date", fmt_date="%d/%m/%Y %H:%M:%S"),
        String("analyte_id"),
        String("result"),
        String("flags"),
        String("fluide"),
        String("reactif_string"),
        String("calibration_string"),
        String("controle_string"),
        String("reactif_ref"),
    )


class QCML(list):
    def add(self, **kwargs):
        obj = QCMLResult(**kwargs)
        self.append(obj)
        return obj

    def parse(self, data):
        for line in data.splitlines():
            if not line:
                continue
            self.append(QCMLResult().parse(line))
        return self

    def to_string(self):
        return "\n".join([e.to_string() for e in self])


class Driver(MessageV2):
    SPEC = {"code": "qcmlqc", "label": "QCML QC"}
    DATE_FORMAT = "%d/%m/%Y %H:%M:%S"

    def create(self, **kwargs):
        return QCML(**kwargs)

    def parse(self, data):
        return QCML().parse(data)

    def to_string(self, message):
        data = message.to_string()
        return data

    def encode_worklist(self, worklist, context):
        msg = self.create()
        # Uniquement les contextes de type result
        if worklist.context != "result":
            return None
        for sample in worklist.samples:
            # Uniquement les samples de type control
            if sample.type != "control":
                continue
            for result in sample.results:
                reagent = ""
                calibration_string = ""
                reactif_string = ""
                for lot in result.lots:
                    if lot.type == "reagent":
                        reagent = lot.lot
                        reactif_string = lot.name or lot.lot
                        if lot.expiration:
                            reactif_string += "^%s" % lot.expiration.strftime(
                                self.DATE_FORMAT
                            )
                    if lot.type == "calibration":
                        calibration_string = lot.name or lot.lot
                        if lot.expiration:
                            calibration_string += "^%s" % lot.expiration.strftime(
                                self.DATE_FORMAT
                            )
                if sample.control_name:
                    nom_lot = sample.control_name
                else:
                    nom_lot = sample.sample_id
                msg.add(
                    instrument_id=result.instrument,
                    lot=sample.sample_id,
                    analyte_id=result.code,
                    date=result.date,
                    result=result.value,
                    fluide=sample.specimen,
                    flags="\\".join(result.flags),
                    reactif_string=reactif_string,
                    calibration_string=calibration_string,
                    controle_string=nom_lot,
                    reactif_ref=reagent,
                )
        if msg:
            return msg
        else:
            return None
